@@ -27,15 +27,24 @@ GtkWidget* construct_view_table(char* tabname)
  int row,col;
  char errstr[MAX_QUERY_LENGTH+100];
  struct result_cache res_cache;
+#ifdef SMALLOC
+ struct dmem_list *memory;
+#else
  struct mem_list *memory;
+#endif
  int i,j,orow,ocol;
  GtkWidget*** table_entry;
  GtkWidget* table_view;
  GtkWidget* scroller_view;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d construct_table_view",__FILE__,__LINE__);
+ debug("construct_table_view: %s", tabname);
  memory = NULL;
  if (!connection) { error("no_connection %s:%d", __FILE__,__LINE__); return NULL; }
+#ifdef SMALLOC
+ dmem_execute_printf(&memory, connection, &ptr, &row, &col, errstr, 1, HERE, "SELECT * FROM %s", tabname);
+#else
  mem_execute_printf(&memory, connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s", tabname);
+#endif
  if (row<0 || col<0) { debug("errstr=%s", errstr); return NULL;	} /* FIXME SHOULDN'T BE THERE DIALOG ?? */
  res_cache.cols = col;
  res_cache.rows = row;
@@ -71,27 +80,33 @@ GtkWidget* construct_view_table(char* tabname)
  gtk_container_set_border_width(GTK_CONTAINER(scroller_view), 1);
  gtk_scrolled_window_set_placement(GTK_SCROLLED_WINDOW(scroller_view), GTK_CORNER_TOP_RIGHT);
  gtk_widget_show(scroller_view);
+#ifdef SMALLOC
+ dmem_free_p3c(&memory, &ptr, orow, ocol, HERE);
+ dshred_special_memlist(&memory, HERE);
+#else
  mem_free_p3c(&memory, &ptr, orow, ocol);
  shred_special_memlist(&memory);
+#endif
  return scroller_view;
 }
 
 int drop_control(GtkWidget* w, gpointer data)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d drop_control",__FILE__,__LINE__);
+ create_pbar("freeing memory...");
  gtk_main_quit();
  return FALSE;
 }
 
 void rotate_book(GtkButton *button,GtkNotebook *notebook)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d rotate_book",__FILE__,__LINE__);
  gtk_notebook_set_tab_pos(notebook,(notebook->tab_pos+1)%4);
 }
 
 void write_html_header(FILE* f)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d write_html_header",__FILE__,__LINE__);
  fprintf(f,"<HTML>\n");
  fprintf(f,"<TITLE>\n");
  fprintf(f,"Results\n");
@@ -103,7 +118,7 @@ void write_html_header(FILE* f)
 
 void write_html_end(FILE* f)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d write_html_end",__FILE__,__LINE__);
  fprintf(f,"</BODY>\n");
  fprintf(f,"</HTML>\n");
 }
@@ -113,12 +128,20 @@ int save_table(FILE* f, char** tables, int page, int ntabs, int fast)
  int start,want_meta,i,j,row,col;
  char errstr[MAX_QUERY_LENGTH+100];
  char*** ptr;
+#ifdef SMALLOC
+ struct dmem_list* mem;
+#else
  struct mem_list* mem;
+#endif
  mem = NULL;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d save_table",__FILE__,__LINE__);
  if (!f) { gtk_dialog_printf("Cannot write to NULL file!"); return 0; }
  if (ntabs<0 || page<0 || page>=ntabs) { gtk_dialog_printf("Cannot save page: %d of %d", page, ntabs); return 0; }
+#ifdef SMALLOC
+ dmem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 1, HERE, "SELECT * FROM %s", tables[page]);
+#else
  mem_execute_printf(&mem, connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s", tables[page]);
+#endif
  if (row<0 || col<0)
    {
     gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
@@ -139,25 +162,41 @@ int save_table(FILE* f, char** tables, int page, int ntabs, int fast)
  fprintf(f,"</BODY>\n");
  fprintf(f,"</HTML>\n");
  if (!fast) fclose(f);
+#ifdef SMALLOC
+ dmem_free_p3c(&mem, &ptr, row, col, HERE);
+ dshred_special_memlist(&mem, HERE);
+#else
  mem_free_p3c(&mem, &ptr, row, col);
  shred_special_memlist(&mem);
+#endif
  return 1;
 }
 
 void save_entire_book(GtkButton *button, GtkNotebook *notebook)
 {
+#ifdef SMALLOC
+ struct dmem_list* memory;
+#else
  struct mem_list* memory;
+#endif
  char** tables;
  char* filen;
  int ntabs,a,i;
+ void* tmp;
  FILE* f;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d save_entire_book",__FILE__,__LINE__);
+ tmp=NULL;
  memory = NULL;
+#ifdef SMALLOC
+ if (!dmem_get_db_tables(&memory, connection,&tables, &ntabs, want_all_elems,HERE)) 
+#else
  if (!mem_get_db_tables(&memory, connection,&tables, &ntabs, want_all_elems)) 
+#endif
     { 
      gtk_dialog_printf("Cannot get DB tables list!\n"); 
      return; 
     }
+ if (ntabs<=0) { gtk_dialog_printf("No tables detected!"); return; }
  filen = gtk_dialog_getfile(".HTML", "Save Results to: (*.HTML)");
  if (!strcmp(filen,""))
    {
@@ -178,32 +217,58 @@ void save_entire_book(GtkButton *button, GtkNotebook *notebook)
     return;
    }
  write_html_header(f);
- fprintf(f,"<CENTER><BIG> DataBase: %s </BIG><?CENTER>\n", currentbase);
- for (i=0;i<ntabs;i++) if (!save_table(f, tables, i, ntabs, 1)) break;
+ fprintf(f,"<CENTER><BIG><BIG> DataBase: %s </BIG></BIG></CENTER>\n", currentbase);
+ for (i=0;i<ntabs;i++) 
+   {
+    fprintf(f, "<CENTER><BIG>table: %s</BIG></CENTER>\n", tables[i]);
+    if (!save_table(f, tables, i, ntabs, 1)) break;
+   }
  write_html_end(f);
  fclose(f);
+#ifdef SMALLOC
+ for (i=0;i<ntabs;i++) if (tables[i]) { dsfree(&memory, (void**)(&tables[i]), HERE); tables[i] = NULL; }
+ if (tables) { dsfree(&memory, (void**)(&tables), HERE); tables = NULL; }
+ dshred_special_memlist(&memory, HERE);
+#else
  for (i=0;i<ntabs;i++) if (tables[i]) { sfree(&memory, (void**)(&tables[i])); tables[i] = NULL; }
- if (tables) { sfree(&memory, (void**)(&tables)); tables = NULL; }
+ if (tables) 
+   {
+    tmp = (void*)tables;
+    sfree(&memory, &tmp); 
+    tables = NULL; 
+   }
  shred_special_memlist(&memory);
+#endif
 }
 
 void save_table_book(GtkButton *button, GtkNotebook *notebook)
 {
  int page;
+#ifdef SMALLOC
+ struct dmem_list* memory;
+#else
  struct mem_list* memory;
+#endif
  char** tables;
  char* filen;
+ void* tmp;
  int ntabs,a,i;
  FILE* f;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d save_table_book",__FILE__,__LINE__);
  memory = NULL;
+ tmp=NULL;
  page = gtk_notebook_get_current_page(notebook);
  debug("PAGE=%d\n",page);
+#ifdef SMALLOC
+ if (!dmem_get_db_tables(&memory, connection,&tables, &ntabs, want_all_elems, HERE)) 
+#else
  if (!mem_get_db_tables(&memory, connection,&tables, &ntabs, want_all_elems)) 
+#endif
     { 
      gtk_dialog_printf("Cannot get DB tables list!\n"); 
      return; 
     }
+ if (ntabs<=0) { gtk_dialog_printf("No tables detected!"); return; }
  filen = gtk_dialog_getfile(".HTML", "Save Results to: (*.HTML)");
  if (!strcmp(filen,""))
    {
@@ -225,19 +290,27 @@ void save_table_book(GtkButton *button, GtkNotebook *notebook)
    }
  save_table(f, tables, page, ntabs, 0);
  fclose(f);
- print_special_memlist(memory);
+#ifdef SMALLOC
+ for (i=0;i<ntabs;i++) if (tables[i]) { dsfree(&memory, (void**)(&tables[i]), HERE); tables[i] = NULL; }
+ if (tables) { dsfree(&memory, (void**)(&tables), HERE); tables = NULL; }
+ dshred_special_memlist(&memory, HERE);
+#else
  for (i=0;i<ntabs;i++) if (tables[i]) { sfree(&memory, (void**)(&tables[i])); tables[i] = NULL; }
- if (tables) { sfree(&memory, (void**)(&tables)); tables = NULL; }
- print_special_memlist(memory);
+ if (tables) 
+   { 
+    tmp = (void*)tables;
+    sfree(&memory, &tmp); 
+    tables = NULL; 
+   }
  shred_special_memlist(&memory);
- print_special_memlist(memory);
+#endif
 }
 
 void tabsborder_book(GtkButton *button, GtkNotebook *notebook)
 {
  int tval = FALSE;
  int bval = FALSE;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d tabsborder_book",__FILE__,__LINE__);
  if (notebook->show_tabs == 0)   tval = TRUE; 
  if (notebook->show_border == 0) bval = TRUE;
  gtk_notebook_set_show_tabs(notebook, tval);
@@ -246,14 +319,14 @@ void tabsborder_book(GtkButton *button, GtkNotebook *notebook)
 
 void create_pbar(char* arg)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d create_pbar",__FILE__,__LINE__);
  cont=1;
  system_printf("pbar %d '%s' 2>/dev/null &", getpid(), arg);
 }
 
 void destroy_pbar()
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d destroy_pbar",__FILE__,__LINE__);
  system("killall -9 pbar 2>/dev/null");
  cont=1;
 }
@@ -268,14 +341,29 @@ void cmenu_browse_all(GtkWidget* w, gpointer data)
  GtkWidget *label;
  GtkWidget *scroller; 
  GtkWidget *vtable;
+#ifdef SMALLOC
+ struct dmem_list* memoryl;
+#else
  struct mem_list* memoryl;
+#endif
  char** tables;
+ void*  tmp;
  int ntabs;
  int i;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d cmenu_browse_all",__FILE__,__LINE__);
  memoryl=NULL;
+ tmp=NULL;
  if (!connected)  { gtk_dialog_printf("Not connected to DB!"); return; }
- if (!mem_get_db_tables(&memoryl, connection,&tables, &ntabs, want_all_elems)) { gtk_dialog_printf("Cannot get DB tables list!\n"); return; }
+#ifdef SMALLOC
+ if (!dmem_get_db_tables(&memoryl, connection,&tables, &ntabs, want_all_elems, HERE)) 
+#else
+ if (!mem_get_db_tables(&memoryl, connection,&tables, &ntabs, want_all_elems)) 
+#endif
+   { 
+    gtk_dialog_printf("Cannot get DB tables list!\n"); 
+    return; 
+   }
+ if (ntabs<=0) { gtk_dialog_printf("No tables detected!"); return; }
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize(window, 800,600);
  gtk_window_set_title (GTK_WINDOW(window), "Preview of DB (results will be truncated if too long)");
@@ -358,7 +446,25 @@ void cmenu_browse_all(GtkWidget* w, gpointer data)
 /* gtk_window_set_title (GTK_WINDOW(window), "Preview of DB");*/
  gtk_main();
  if (GTK_IS_WIDGET(window)) gtk_widget_destroy(window);
- for (i=0;i<ntabs;i++) if (tables[i]) { sfree(&memoryl, (void**)(&tables[i])); tables[i]=NULL; }
- if (tables) { sfree(&memoryl, (void**)(&tables)); tables=NULL; }
+#ifdef SMALLOC
+ for (i=0;i<ntabs;i++) if (tables[i]) { dsfree(&memoryl, (void**)(&tables[i]), HERE); tables[i]=NULL; }
+ if (tables) 
+   { dsfree(&memoryl, (void**)(&tables), HERE); tables=NULL; }
+ dshred_special_memlist(&memoryl, HERE);
+#else
+ for (i=0;i<ntabs;i++) if (tables[i]) 
+   { 
+    sfree(&memoryl, (void**)(&tables[i])); 
+    tables[i]=NULL; 
+   }
+ if (tables) 
+   { 
+    tmp = (void*)tables;
+    sfree(&memoryl, &tmp); 
+    tables=NULL; 
+   }
  shred_special_memlist(&memoryl);
+#endif
+ destroy_pbar();
 }
+
