@@ -18,7 +18,7 @@ static int col = 0;
 int strhasdot(char* str)
 {
  int l,i;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d strhasdot",__FILE__,__LINE__);
  if (!str) return 0;
  l = strlen(str)-1;
  for (i=l;i>=0;i--)
@@ -30,13 +30,17 @@ int strhasdot(char* str)
  return 0;
 }
 
+void destroy_widget(GtkWidget* w)
+{
+ if (GTK_IS_WIDGET(w)) gtk_widget_destroy(w);
+}
 
 int mask_match()
 {
  char tmps[MAX_ENTRY_LENGTH];
  char tmpm[MASK_LENGTH];
  int i,j;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d mask_match",__FILE__,__LINE__);
  strcpy(tmps, tentry);
  strcpy(tmpm, fmask);
  downcase(tmps);
@@ -65,37 +69,37 @@ int mask_match()
 
 void destroy_fs(GtkWidget* w, gpointer d)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d destroy_fs",__FILE__,__LINE__);
  gtk_main_quit();
  if (!d) return;
- gtk_widget_destroy((GtkWidget*)(d));
+ destroy_widget((GtkWidget*)(d));
 }
 
 
 void cancel_fs(GtkWidget* w, gpointer d)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d cancel_fs",__FILE__,__LINE__);
  gtk_main_quit();
  if (!d) return;
- gtk_widget_destroy((GtkWidget*)(d));
+ destroy_widget((GtkWidget*)(d));
  strcpy(tentry,"");
 }
 
 
 void ok_fs(GtkWidget* w, gpointer d)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d ok_fs",__FILE__,__LINE__);
  if (!d) return;
  strcpy(tentry,gtk_file_selection_get_filename((GTK_FILE_SELECTION(d))));
  if (!mask_match()) strcpy(tentry,"");
  gtk_main_quit();
- gtk_widget_destroy((GtkWidget*)(d));
+ destroy_widget((GtkWidget*)(d));
 }
 
 
 gint delete_event_proc()
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d delete_event_proc",__FILE__,__LINE__);
  answer = ANSWER_NO;
  strcpy(tentry, "");
  gtk_main_quit();
@@ -105,7 +109,7 @@ gint delete_event_proc()
 
 gint delete_combos_proc(GtkWidget* w, gpointer d)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d delete_combos_proc",__FILE__,__LINE__);
  answer = ANSWER_NO;
  strcpy(tentry, "");
  gtk_main_quit();
@@ -117,16 +121,16 @@ gint delete_combos_proc(GtkWidget* w, gpointer d)
 
 void ok_proc(GtkWidget* w, gpointer d)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d ok_proc",__FILE__,__LINE__);
  gtk_main_quit();
  if (!d) return;
- gtk_widget_destroy((GtkWidget*)(d));
+ destroy_widget((GtkWidget*)(d));
 }
 
 
 void ok_combo_proc(GtkWidget* w, gpointer cmb)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d ok_combo_proc",__FILE__,__LINE__);
  if (!cmb) return;
  strcpy(tentry,gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(cmb)->entry)));
  gtk_main_quit();
@@ -137,38 +141,50 @@ void ok_combo_proc(GtkWidget* w, gpointer cmb)
 
 void entry_changed_proc(GtkWidget* w, gpointer d)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d entry_changed_proc",__FILE__,__LINE__);
  strcpy(tentry,gtk_entry_get_text(GTK_ENTRY(w)));
 }
 
 
 void entry_proc(GtkWidget* w, gpointer d)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d entry_proc",__FILE__,__LINE__);
  strcpy(tentry,gtk_entry_get_text(GTK_ENTRY(w)));
  gtk_main_quit();
  if (!d) return;
- gtk_widget_destroy((GtkWidget*)(d));
+ destroy_widget((GtkWidget*)(d));
 }
 
 
 void yes_proc(GtkWidget* w, gpointer d)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d yes_proc",__FILE__,__LINE__);
  answer = ANSWER_YES;
  gtk_main_quit();
  if (!d) return;
- gtk_widget_destroy((GtkWidget*)(d));
+ destroy_widget((GtkWidget*)(d));
+}
+
+void return_value(GtkWidget* w, gpointer ptr)
+{
+ GtkAdjustment* adj;
+ struct carrier* car;
+ car = (struct carrier*)ptr;
+ debug("%s:%d yes_proc",__FILE__,__LINE__);
+ adj = gtk_range_get_adjustment(GTK_RANGE(&(GTK_SCALE(car->ptr)->range)));
+ answer = (int)(adj->value);
+ gtk_main_quit();
+ destroy_widget(car->widget);
 }
 
 
 void no_proc(GtkWidget* w, gpointer d)
 {
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d no_proc",__FILE__,__LINE__);
  answer = ANSWER_NO;
  gtk_main_quit();
  if (!d) return;
- gtk_widget_destroy((GtkWidget*)(d));
+ destroy_widget((GtkWidget*)(d));
 }
 
 
@@ -176,7 +192,7 @@ char* gtk_dialog_getfile_printf(const char* mask, const char* fmt, ...)
 {
  va_list ap;
  char msg[MAX_MSG_LENGTH];
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_getfile_printf",__FILE__,__LINE__);
  if (!fmt) return NULL;
  va_start(ap, fmt);
  vsprintf(msg,fmt,ap);
@@ -189,7 +205,7 @@ char* gtk_dialog_gettext_printf(const char* fmt, ...)
 {
  va_list ap;
  char msg[MAX_MSG_LENGTH];
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_gettext_printf",__FILE__,__LINE__);
  if (!fmt) return NULL;
  va_start(ap, fmt);
  vsprintf(msg,fmt,ap);
@@ -202,7 +218,7 @@ char* gtk_dialog_gettext_big_printf(const char* fmt, ...)
 {
  va_list ap;
  char msg[MAX_MSG_LENGTH];
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_gettext_big_printf",__FILE__,__LINE__);
  if (!fmt) return NULL;
  va_start(ap, fmt);
  vsprintf(msg,fmt,ap);
@@ -210,11 +226,60 @@ char* gtk_dialog_gettext_big_printf(const char* fmt, ...)
  return gtk_dialog_gettext_big(msg);
 }
 
+int gtk_dialog_getnum_printf(int a, int b, char* fmt, ...)
+{
+ va_list ap;
+ char msg[MAX_MSG_LENGTH];
+ debug("%s:%d gtk_dialog_getnum_printf",__FILE__,__LINE__);
+ if (!fmt) return -1;
+ va_start(ap, fmt);
+ vsprintf(msg,fmt,ap);
+ va_end(ap);
+ return gtk_dialog_getnum(a,b,msg);
+}
+
+int gtk_dialog_getnum(int ua, int ub, char* msg)
+{
+ GtkWidget *scale, *box, *window, *button;
+ GtkObject *adj;
+ struct carrier* car;
+ int a,b;
+ debug("%s:%d gtk_dialog_getnum",__FILE__,__LINE__);
+ a = (ua<=ub)?ua:ub;
+ b = (ua>ub) ?ua:ub;
+ if (a==b) return a;
+ window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+ gtk_window_set_modal(GTK_WINDOW(window), TRUE);
+ gtk_widget_set_usize(window,500,100);
+ gtk_window_set_title (GTK_WINDOW(window), msg);
+ box = gtk_vbox_new(FALSE, 0);
+ gtk_container_add(GTK_CONTAINER(window), box);
+ gtk_container_set_border_width(GTK_CONTAINER(box), 10);
+ gtk_widget_show(box);
+ adj = gtk_adjustment_new(0.0,(gfloat)(a),(gfloat)(b)+1.0, 1.0, 1.0, 1.0);
+ scale = gtk_hscale_new(GTK_ADJUSTMENT(adj));
+ gtk_scale_set_digits(GTK_SCALE(scale), 0);
+ gtk_scale_set_draw_value(GTK_SCALE(scale), TRUE);
+ gtk_box_pack_start(GTK_BOX(box), scale, TRUE, TRUE, 0);
+ button = gtk_button_new_with_label("OK");
+ car = (struct carrier*)malloc(sizeof(struct carrier));
+ car->widget = window;
+ car->ptr    = (void*)scale; 
+ gtk_signal_connect (GTK_OBJECT(window), "destroy",GTK_SIGNAL_FUNC(return_value),(gpointer)(car));
+ gtk_signal_connect(GTK_OBJECT(button), "clicked",GTK_SIGNAL_FUNC(return_value),(gpointer)(car));
+ gtk_widget_show(button);
+ gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
+ gtk_widget_show(scale);
+ gtk_widget_show(window);
+ gtk_main();
+ free(car);
+ return answer;
+}
 
 char* gtk_dialog_getfile(const char* mask, const char* question)
 {
  GtkWidget* filew, *flist;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_getfile",__FILE__,__LINE__);
  strcpy(fmask,"");
  if (!question)  filew = gtk_file_selection_new("NULL");
  if (strlen(mask)>=MASK_LENGTH) { error("mask too long %s:%d", __FILE__,__LINE__); return NULL; }
@@ -237,7 +302,7 @@ char* gtk_dialog_getfile(const char* mask, const char* question)
 char* gtk_dialog_gettext_big(const char* question)
 {
  GtkWidget* dialog, *label, *button, *entry;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_gettext_big",__FILE__,__LINE__);
  if (!question) warn("question=NULL: %s:%d",__FILE__,__LINE__);
  strcpy(tentry, "");
  dialog = gtk_dialog_new();
@@ -247,17 +312,17 @@ char* gtk_dialog_gettext_big(const char* question)
  label = gtk_label_new(question);
  entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry), MAX_ENTRY_LENGTH-2);
- gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(entry_proc), (gpointer)dialog);
- gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(entry_changed_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), entry, TRUE, TRUE, 0);
  gtk_widget_show(label);
  gtk_widget_show(entry);
  gtk_widget_grab_focus(entry);
  button = gtk_button_new_with_label("OK");
- gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(entry_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(entry_changed_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_widget_show(button);
  gtk_widget_show(dialog);
  gtk_main();
@@ -268,7 +333,7 @@ char* gtk_dialog_gettext_big(const char* question)
 char* gtk_dialog_gettext(const char* question)
 {
  GtkWidget* dialog, *label, *button, *entry;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_gettext",__FILE__,__LINE__);
  if (!question) warn("question=NULL: %s:%d",__FILE__,__LINE__);
  strcpy(tentry, "");
  dialog = gtk_dialog_new();
@@ -278,17 +343,17 @@ char* gtk_dialog_gettext(const char* question)
  label = gtk_label_new(question);
  entry = gtk_entry_new();
  gtk_entry_set_max_length(GTK_ENTRY(entry), MAX_ENTRY_LENGTH-2);
- gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(entry_proc), (gpointer)dialog);
- gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(entry_changed_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), entry, TRUE, TRUE, 0);
  gtk_widget_show(label);
  gtk_widget_show(entry);
  gtk_widget_grab_focus(entry);
  button = gtk_button_new_with_label("OK");
- gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(entry_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(entry_changed_proc), (gpointer)dialog);
  gtk_widget_show(button);
  gtk_widget_show(dialog);
  gtk_main();
@@ -300,7 +365,7 @@ void gtk_dialog_printf(const char* fmt, ...)
 {
  va_list ap;
  char msg[MAX_MSG_LENGTH];
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_printf",__FILE__,__LINE__);
  if (!fmt) return ;
  va_start(ap, fmt);
  vsprintf(msg,fmt,ap);
@@ -315,7 +380,7 @@ void gtk_dialog_printf_big_with_title(const char* title, const char* fmt, ...)
  GtkWidget* dialog, *label, *button;
  char msg[MAX_MSG_LENGTH];
  va_list ap;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_printf_big_with_title",__FILE__,__LINE__);
  if (!title) warn("title=NULL: %s:%d",__FILE__,__LINE__);
  va_start(ap,fmt);
  vsprintf(msg,fmt,ap);
@@ -328,9 +393,9 @@ void gtk_dialog_printf_big_with_title(const char* title, const char* fmt, ...)
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
  gtk_widget_show(label);
  button = gtk_button_new_with_label("OK");
- gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_widget_grab_focus(button);
  gtk_widget_show(button);
  gtk_widget_show(dialog);
@@ -343,7 +408,7 @@ void gtk_dialog_printf_big(const char* fmt, ...)
  GtkWidget* dialog, *label, *button;
  char msg[MAX_MSG_LENGTH];
  va_list ap;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_big_printf",__FILE__,__LINE__);
  if (!fmt) return ;
  va_start(ap,fmt);
  vsprintf(msg,fmt,ap);
@@ -356,9 +421,9 @@ void gtk_dialog_printf_big(const char* fmt, ...)
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
  gtk_widget_show(label);
  button = gtk_button_new_with_label("OK");
- gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_widget_grab_focus(button);
  gtk_widget_show(button);
  gtk_widget_show(dialog);
@@ -371,7 +436,7 @@ void gtk_dialog_printf_with_title(const char* title, const char* fmt, ...)
  GtkWidget* dialog, *label, *button;
  char msg[MAX_MSG_LENGTH];
  va_list ap;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_printf_with_title",__FILE__,__LINE__);
  if (!fmt) return;
  va_start(ap,fmt);
  vsprintf(msg,fmt,ap);
@@ -384,9 +449,9 @@ void gtk_dialog_printf_with_title(const char* title, const char* fmt, ...)
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
  gtk_widget_show(label);
  button = gtk_button_new_with_label("OK");
- gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_widget_grab_focus(button);
  gtk_widget_show(button);
  gtk_widget_show(dialog);
@@ -397,7 +462,7 @@ void gtk_dialog_printf_with_title(const char* title, const char* fmt, ...)
 void gtk_dialog_mbox(const char* title, const char* msg)
 {
  GtkWidget* dialog, *label, *button;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_mbox",__FILE__,__LINE__);
  if (!msg || !title) return;
  dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(dialog), title);
@@ -407,9 +472,9 @@ void gtk_dialog_mbox(const char* title, const char* msg)
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
  gtk_widget_show(label);
  button = gtk_button_new_with_label("OK");
- gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
  gtk_widget_grab_focus(button);
  gtk_widget_show(button);
  gtk_widget_show(dialog);
@@ -422,7 +487,7 @@ int gtk_dialog_yes_no_printf(const char* fmt, ...)
  GtkWidget* dialog, *label, *button_yes, *button_no;
  char question[MAX_MSG_LENGTH];
  va_list ap;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_yes_no_printf",__FILE__,__LINE__);
  if (!fmt) return 0;
  va_start(ap,fmt);
  vsprintf(question,fmt,ap);
@@ -436,11 +501,11 @@ int gtk_dialog_yes_no_printf(const char* fmt, ...)
  gtk_widget_show(label);
  button_yes = gtk_button_new_with_label("Yes");
  button_no  = gtk_button_new_with_label("No");
- gtk_signal_connect(GTK_OBJECT(button_yes), "clicked", GTK_SIGNAL_FUNC(yes_proc), (gpointer)dialog);
- gtk_signal_connect(GTK_OBJECT(button_no),  "clicked", GTK_SIGNAL_FUNC(no_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button_yes, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button_no, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button_yes), "clicked", GTK_SIGNAL_FUNC(yes_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button_no),  "clicked", GTK_SIGNAL_FUNC(no_proc), (gpointer)dialog);
  gtk_widget_grab_focus(button_yes);
  gtk_widget_show(button_yes);
  gtk_widget_show(button_no);
@@ -453,7 +518,7 @@ int gtk_dialog_yes_no_printf(const char* fmt, ...)
 int gtk_dialog_with_2_buttons(const char* question, char* but1, char* but2)
 {
  GtkWidget* dialog, *label, *button_yes, *button_no;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_with_2_buttons",__FILE__,__LINE__);
  if (!question || !but1 || !but2) { warn("parameters! %s:%d", __FILE__,__LINE__); return 0; }
  dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(dialog), "Confirmation needed.");
@@ -464,11 +529,11 @@ int gtk_dialog_with_2_buttons(const char* question, char* but1, char* but2)
  gtk_widget_show(label);
  button_yes = gtk_button_new_with_label(but1);
  button_no  = gtk_button_new_with_label(but2);
- gtk_signal_connect(GTK_OBJECT(button_yes), "clicked", GTK_SIGNAL_FUNC(yes_proc), (gpointer)dialog);
- gtk_signal_connect(GTK_OBJECT(button_no),  "clicked", GTK_SIGNAL_FUNC(no_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button_yes, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button_no, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button_yes), "clicked", GTK_SIGNAL_FUNC(yes_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button_no),  "clicked", GTK_SIGNAL_FUNC(no_proc), (gpointer)dialog);
  gtk_widget_grab_focus(button_yes);
  gtk_widget_show(button_yes);
  gtk_widget_show(button_no);
@@ -481,7 +546,7 @@ int gtk_dialog_with_2_buttons(const char* question, char* but1, char* but2)
 int gtk_dialog_yes_no(const char* question)
 {
  GtkWidget* dialog, *label, *button_yes, *button_no;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_yes_no",__FILE__,__LINE__);
  if (!question) { warn("question=NULL, %s:%d", __FILE__,__LINE__); return 0; }
  dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(dialog), "Confirmation needed.");
@@ -492,11 +557,11 @@ int gtk_dialog_yes_no(const char* question)
  gtk_widget_show(label);
  button_yes = gtk_button_new_with_label("Yes");
  button_no  = gtk_button_new_with_label("No");
- gtk_signal_connect(GTK_OBJECT(button_yes), "clicked", GTK_SIGNAL_FUNC(yes_proc), (gpointer)dialog);
- gtk_signal_connect(GTK_OBJECT(button_no),  "clicked", GTK_SIGNAL_FUNC(no_proc), (gpointer)dialog);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button_yes, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button_no, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button_yes), "clicked", GTK_SIGNAL_FUNC(yes_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button_no),  "clicked", GTK_SIGNAL_FUNC(no_proc), (gpointer)dialog);
  gtk_widget_grab_focus(button_yes);
  gtk_widget_show(button_yes);
  gtk_widget_show(button_no);
@@ -511,7 +576,7 @@ int setup_combo(PGconn* c, GtkWidget* w, GtkWidget** cmb)
  char errstr[MAX_QUERY_LENGTH];
  int i;
  GList* list;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d setup_combo",__FILE__,__LINE__);
  list = NULL;
  if (!c || !w) error("setup_combo, %s:%d",__FILE__,__LINE__);
  ptr=p3c_execute_printf(c,&row,&col,errstr, 0, "SELECT datname FROM pg_database");
@@ -537,7 +602,7 @@ char* gtk_dialog_getdbs(PGconn* connection, char* fmt, ...)
  GtkWidget* dialog, *combo, *button, *label;
  char msg[MAX_MSG_LENGTH];
  va_list ap;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d gtk_dialog_getdbs",__FILE__,__LINE__);
  if (!connection || !fmt) error("gtk_getdbs, params: %s:%d", __FILE__,__LINE__);
  va_start(ap,fmt);
  vsprintf(msg,fmt,ap);
@@ -553,16 +618,16 @@ char* gtk_dialog_getdbs(PGconn* connection, char* fmt, ...)
  button = gtk_button_new_with_label("Select");
  combo=NULL;
  if (setup_combo(connection, dialog, &combo)==-1) return NULL;
- gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_combo_proc), (gpointer)combo);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), combo, TRUE, TRUE, 0);
  gtk_widget_show(combo);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_combos_proc),(gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_combo_proc), (gpointer)combo);
  gtk_widget_grab_focus(combo);
  gtk_widget_show(button);
  gtk_widget_show(dialog);
  gtk_main();
- gtk_widget_destroy(dialog);
+ destroy_widget(dialog);
  return tentry;
 }
 
