@@ -12,6 +12,8 @@
  FILE: main.h */
 #ifndef __MORGOTHDBMA_MAIN_H__
 #define __MORGOTHDBMA_MAIN_H__
+#define GTK_TAB_KEY  65289
+
 #include "gtkincl.h"
 #include "consoleout.h"
 #include "gtkdialogs.h"
@@ -19,6 +21,7 @@
 #include "toolbar.h"
 #include "entireDB.h"
 #include "mmsystem.h"
+#include "sqlparser.h"
 void disconnect();
 int dummy_connect(int, char*);
 void __dummy( GtkWidget*,gpointer);
@@ -64,6 +67,11 @@ void construct_components();
 void browse_table(char* tabname);
 void destruct_tab_view(GtkWidget* w, gpointer data);
 void post_start(GtkWidget* w, gpointer data);
+void clear_wname();
+void set_wname_tab(char* arg);
+void destroy_global_auto_complete();
+void process_parser(GtkWidget*);
+
 struct result_cache
 {
  char*** data;
