@@ -34,9 +34,10 @@ struct main_components
  GtkWidget* button_destroy;
  GtkWidget*** table_entry;
  GtkWidget* last_selected;
- int ntables,nviews,ntriggers;
+ int ntables,nviews,ntriggers,cols,rows,acols,arows,rrow,rcol;
+ /* cols HOW MUCH ON WINDOW, acols ALL COLUMNS, RCOL, ROTATE_COL, OFFSET COL ON WINDOW */
  char selected[STDSTRING];
- char seltype,cols,rows;
+ char seltype;
  char*** data;
 };
 
