@@ -12,14 +12,14 @@
 #include "query.h"
 int timeout=10;
 int cont=1;
+PGconn* connection;
 
 int main(int lb, char** par)
 {
- PGconn* connection;
  PGresult* pgres;
  char*** out;
  int a,b;
- debug("%s:%d",__FILE__,__LINE__);
+ debug("%s:%d pgxtest",__FILE__,__LINE__);
  connection=0;
  pgres=0;
  out=0;
