@@ -7,7 +7,6 @@
  Currently only one author: MOrgoth DBMA
  FILE: query.c */
 #include "common.h"
-#include <libpq-fe.h>
 #include "consoleout.h"
 #include "query.h"
 #include "signals.h"
@@ -16,11 +15,12 @@
 int ping_conn;
 extern int timeout;
 
+
 int connAlive(PGconn* c)
 {
  PGresult* r;
  int stat;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d connAlive", __FILE__,__LINE__);
  ping_conn=1;
  alarm_clock(timeout);
  r= PQexec(c, "SELECT datname FROM pg_database");
@@ -42,7 +42,7 @@ int connAlive(PGconn* c)
 
 void disconnect_db(PGconn** c)
 {
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d disconnect_db", __FILE__,__LINE__);
  if (*c) PQfinish(*c);
  else warn("connect is null: %s:%d", __FILE__,__LINE__);
  *c = NULL;
@@ -54,7 +54,7 @@ PGconn* connect_db(char* arg)
  PGconn* conn;
  char* strarg;
  conn=0;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d connect_db", __FILE__,__LINE__);
  strarg = (char*)malloc(strlen(arg)+32);
  debug("\nconnect to database %s\n", arg);
  sprintf(strarg,"dbname=template1");
@@ -86,7 +86,7 @@ PGconn* gtk_connect_db(char* arg, char* errstr)
  PGconn* conn;
  char* strarg;
  conn=0;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d gtk_connect_db", __FILE__,__LINE__);
  strarg = (char*)malloc(strlen(arg)+16);
  sprintf(strarg,"dbname=template1");
  alarm_clock(timeout);
@@ -113,7 +113,7 @@ PGconn* gtk_connect_db(char* arg, char* errstr)
 
 void clear_result(PGresult** r)
 {
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d clear_result", __FILE__,__LINE__);
  if (*r) PQclear(*r);
  *r=0;
 }
@@ -123,7 +123,7 @@ PGresult* execute_silent_query(PGconn* c,  char* query)
 {
  PGresult* r;
  int stat;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d execute_silent_query", __FILE__,__LINE__);
  debug("SQLquery: \"%s\"\n", query);
  if (!connAlive(c)) { debug("connection dead!\n"); return NULL; }
  if (!query)
@@ -154,7 +154,7 @@ char*** p3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want_
  va_list ap;
  char*** ret;
  char msg[MAX_QUERY_LENGTH*2];
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d p3c_execute_printf", __FILE__,__LINE__);
  if (!c) { error("no connection"); return NULL; }
  va_start(ap, fmt);
  vsprintf(msg,fmt,ap);
@@ -163,12 +163,40 @@ char*** p3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want_
  return ret;
 }
 
+char*** dmp3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want_meta, char* file, int line, char* fmt,...)
+{
+ va_list ap;
+ char*** ret;
+ char msg[MAX_QUERY_LENGTH*2];
+ debug("%s:%d dmp3c_execute_printf called from %s:%d", __FILE__,__LINE__,file,line);
+ if (!c) { error("no connection"); return NULL; }
+ va_start(ap, fmt);
+ vsprintf(msg,fmt,ap);
+ va_end(ap);
+ dmexecute_printf(c, &ret, row, col, errstr, want_meta, HERE, msg);
+ return ret;
+}
+
+char*** dmem_p3c_execute_printf(struct dmem_list** mem, PGconn* c,int* row, int* col, char* errstr, int want_meta, char* file, int line, char* fmt,...)
+{
+ va_list ap;
+ char*** ret;
+ char msg[MAX_QUERY_LENGTH*2];
+ debug("%s:%d dmem_p3c_execute_printf called from %s:%d", __FILE__,__LINE__,file,line);
+ if (!c) { error("no connection"); return NULL; }
+ va_start(ap, fmt);
+ vsprintf(msg,fmt,ap);
+ va_end(ap);
+ dmem_execute_printf(mem, c, &ret, row, col, errstr, want_meta, HERE, msg);
+ return ret;
+}
+
 char*** mp3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want_meta, char* fmt,...)
 {
  va_list ap;
  char*** ret;
  char msg[MAX_QUERY_LENGTH*2];
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d mp3c_execute_printf", __FILE__,__LINE__);
  if (!c) { error("no connection"); return NULL; }
  va_start(ap, fmt);
  vsprintf(msg,fmt,ap);
@@ -182,7 +210,7 @@ char*** mem_p3c_execute_printf(struct mem_list** mem, PGconn* c,int* row, int* c
  va_list ap;
  char*** ret;
  char msg[MAX_QUERY_LENGTH*2];
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d mem_p3c_execute_printf", __FILE__,__LINE__);
  if (!c) { error("no connection"); return NULL; }
  va_start(ap, fmt);
  vsprintf(msg,fmt,ap);
@@ -191,13 +219,60 @@ char*** mem_p3c_execute_printf(struct mem_list** mem, PGconn* c,int* row, int* c
  return ret;
 }
 
+void dmem_execute_printf(struct dmem_list** mem, PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* file, int line, char* fmt,...)
+{
+ int stat;
+ va_list ap;
+ char query[MAX_QUERY_LENGTH*2];
+ PGresult* r;
+ debug("%s:%d mem_execute_printf called from %s:%d", __FILE__,__LINE__,file,line);
+ va_start(ap, fmt);
+ vsprintf(query,fmt,ap);
+ va_end(ap);
+ r=NULL;
+ if (!c)
+   {
+    *res = NULL;
+    *row = *col = -1;
+    strcpy(errstr, "Not connected to DB");
+    return;
+   }
+ if (!query)
+   {
+    *res = NULL;
+    *row = *col = -1;
+    strcpy(errstr, "NULL query passed to DB");
+    return;
+   }
+ if (!connAlive(c)) { sprintf(errstr,"connection dead!\n"); return ; }
+ r= PQexec(c, query);
+ if (!r)
+   {
+    sprintf(errstr,"PQresult is NULL %s\nQuery was:\n%s\n", PQerrorMessage(c), query);
+    *res = NULL;
+    *row = *col = -1;
+    return ;
+   }
+ stat = PQresultStatus(r);
+ if (stat!=PGRES_COMMAND_OK && stat!=PGRES_TUPLES_OK)
+   {
+    sprintf(errstr,"PQexec:\n%s\n%s\nQuery was:\n%s",PQcmdStatus(r),PQresultErrorMessage(r), query);
+    *res = NULL;
+    *row = *col = -1;
+    clear_result(&r);
+    return;
+   }
+ *res = dmem_pgres2pc3(mem, r, row, col, want_meta, HERE);
+ clear_result(&r);
+}
+
 void mem_execute_printf(struct mem_list** mem, PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* fmt,...)
 {
  int stat;
  va_list ap;
  char query[MAX_QUERY_LENGTH*2];
  PGresult* r;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d mem_execute_printf", __FILE__,__LINE__);
  va_start(ap, fmt);
  vsprintf(query,fmt,ap);
  va_end(ap);
@@ -238,13 +313,60 @@ void mem_execute_printf(struct mem_list** mem, PGconn* c, char**** res, int* row
  clear_result(&r);
 }
 
+void dmexecute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* file, int line, char* fmt,...)
+{
+ int stat;
+ va_list ap;
+ char query[MAX_QUERY_LENGTH*2];
+ PGresult* r;
+ debug("%s:%d dmexecute_printf called from %s:%d", __FILE__,__LINE__,file,line);
+ va_start(ap, fmt);
+ vsprintf(query,fmt,ap);
+ va_end(ap);
+ r=NULL;
+ if (!c)
+   {
+    *res = NULL;
+    *row = *col = -1;
+    strcpy(errstr, "Not connected to DB");
+    return;
+   }
+ if (!query)
+   {
+    *res = NULL;
+    *row = *col = -1;
+    strcpy(errstr, "NULL query passed to DB");
+    return;
+   }
+ if (!connAlive(c)) { sprintf(errstr,"connection dead!\n"); return ; }
+ r= PQexec(c, query);
+ if (!r)
+   {
+    sprintf(errstr,"PQresult is NULL %s\nQuery was:\n%s\n", PQerrorMessage(c), query);
+    *res = NULL;
+    *row = *col = -1;
+    return ;
+   }
+ stat = PQresultStatus(r);
+ if (stat!=PGRES_COMMAND_OK && stat!=PGRES_TUPLES_OK)
+   {
+    sprintf(errstr,"PQexec:\n%s\n%s\nQuery was:\n%s",PQcmdStatus(r),PQresultErrorMessage(r), query);
+    *res = NULL;
+    *row = *col = -1;
+    clear_result(&r);
+    return;
+   }
+ *res = dmpgres2pc3(r, row, col, want_meta, HERE);
+ clear_result(&r);
+}
+
 void mexecute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* fmt,...)
 {
  int stat;
  va_list ap;
  char query[MAX_QUERY_LENGTH*2];
  PGresult* r;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d mexecute_printf", __FILE__,__LINE__);
  va_start(ap, fmt);
  vsprintf(query,fmt,ap);
  va_end(ap);
@@ -291,7 +413,7 @@ void execute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, i
  va_list ap;
  char query[MAX_QUERY_LENGTH*2];
  PGresult* r;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d execute_printf", __FILE__,__LINE__);
  va_start(ap, fmt);
  vsprintf(query,fmt,ap);
  va_end(ap);
@@ -338,7 +460,7 @@ void execute_printf_query(PGconn* c,PGresult** r, char* errstr, char* fmt,...)
  int stat;
  va_list ap;
  char query[MAX_QUERY_LENGTH*2];
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d execute_printf_query", __FILE__,__LINE__);
  va_start(ap, fmt);
  vsprintf(query,fmt,ap);
  va_end(ap);
@@ -378,14 +500,15 @@ int get_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems)
  int row,col,i;
  char errstr[MAX_QUERY_LENGTH+100];
  data = *dat;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d get_db_tables", __FILE__,__LINE__);
  if (!want_all_elems)
  execute_printf(c, &res, &row, &col, errstr, 0,
 		 "SELECT tablename from pg_tables WHERE tablename NOT LIKE 'pg_%%' AND tablename NOT LIKE 'sql_%%' ORDER BY tablename");
  else
  execute_printf(c, &res, &row, &col, errstr, 0,
 		 "SELECT tablename from pg_tables ORDER BY tablename");
- if (!res || col<0 || row<0) { printf("errstr=%s\n", errstr); (*data) = NULL; *ntabs=-1; return 0; }
+ if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
+ if (col<0 || row<0) { printf("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
  *ntabs = row;
  data = (char**)malloc(row<<2);
  for (i=0;i<row;i++) 
@@ -393,6 +516,35 @@ int get_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems)
     data[i] = malloc(strlen(res[i][0])+1); 
     strcpy(data[i], res[i][0]); 
    }
+ free_p3c(&res, row, col);		/* FIXME CHECK IT */
+ *dat = data;
+ return 1;
+}
+
+int dmget_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems, char* file, int line)
+{
+ char*** res;
+ char** data;
+ int row,col,i;
+ char errstr[MAX_QUERY_LENGTH+100];
+ data = *dat;
+ debug("%s:%d mget_db_tables called from %s:%d", __FILE__,__LINE__,file,line);
+ if (!want_all_elems)
+ dmexecute_printf(c, &res, &row, &col, errstr, 0,HERE,
+		 "SELECT tablename from pg_tables WHERE tablename NOT LIKE 'pg_%%' AND tablename NOT LIKE 'sql_%%' ORDER BY tablename");
+ else
+ dmexecute_printf(c, &res, &row, &col, errstr, 0, HERE,
+		 "SELECT tablename from pg_tables ORDER BY tablename");
+ if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
+ if (col<0 || row<0) { printf("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
+ *ntabs = row;
+ data = (char**)dmmalloc(row<<2, HERE);
+ for (i=0;i<row;i++) 
+   { 
+    data[i] = dmmalloc(strlen(res[i][0])+1, HERE); 
+    strcpy(data[i], res[i][0]); 
+   }
+ dmfree_p3c(&res, row, col, HERE);		/* FIXME CHECK IT */
  *dat = data;
  return 1;
 }
@@ -404,14 +556,15 @@ int mget_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems)
  int row,col,i;
  char errstr[MAX_QUERY_LENGTH+100];
  data = *dat;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d mget_db_tables", __FILE__,__LINE__);
  if (!want_all_elems)
  execute_printf(c, &res, &row, &col, errstr, 0,
 		 "SELECT tablename from pg_tables WHERE tablename NOT LIKE 'pg_%%' AND tablename NOT LIKE 'sql_%%' ORDER BY tablename");
  else
  execute_printf(c, &res, &row, &col, errstr, 0,
 		 "SELECT tablename from pg_tables ORDER BY tablename");
- if (!res || col<0 || row<0) { printf("errstr=%s\n", errstr); (*data) = NULL; *ntabs=-1; return 0; }
+ if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
+ if (col<0 || row<0) { printf("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
  *ntabs = row;
  data = (char**)mmalloc(row<<2);
  for (i=0;i<row;i++) 
@@ -419,6 +572,7 @@ int mget_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems)
     data[i] = mmalloc(strlen(res[i][0])+1); 
     strcpy(data[i], res[i][0]); 
    }
+ mfree_p3c(&res, row, col);		/* FIXME CHECK IT */
  *dat = data;
  return 1;
 }
@@ -430,14 +584,15 @@ int mem_get_db_tables(struct mem_list** mem, PGconn* c, char*** dat, int* ntabs,
  int row,col,i;
  char errstr[MAX_QUERY_LENGTH+100];
  data = *dat;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d mem_get_db_tables", __FILE__,__LINE__);
  if (!want_all_elems)
  mem_execute_printf(mem,c, &res, &row, &col, errstr, 0,
 		 "SELECT tablename from pg_tables WHERE tablename NOT LIKE 'pg_%%' AND tablename NOT LIKE 'sql_%%' ORDER BY tablename");
  else
  mem_execute_printf(mem,c, &res, &row, &col, errstr, 0,
 		 "SELECT tablename from pg_tables ORDER BY tablename");
- if (!res || col<0 || row<0) { printf("errstr=%s\n", errstr); (*data) = NULL; *ntabs=-1; return 0; }
+ if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
+ if (col<0 || row<0) { debug("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
  *ntabs = row;
  data = (char**)smalloc(mem, row<<2);
  for (i=0;i<row;i++) 
@@ -450,12 +605,40 @@ int mem_get_db_tables(struct mem_list** mem, PGconn* c, char*** dat, int* ntabs,
  return 1;
 }
 
+int dmem_get_db_tables(struct dmem_list** mem, PGconn* c, char*** dat, int* ntabs, int want_all_elems,char* file, int line)
+{
+ char*** res;
+ char** data;
+ int row,col,i;
+ char errstr[MAX_QUERY_LENGTH+100];
+ data = *dat;
+ debug("%s:%d mem_get_db_tables called from %s:%d", __FILE__,__LINE__,file,line);
+ if (!want_all_elems)
+ dmem_execute_printf(mem,c, &res, &row, &col, errstr, 0,HERE,
+		 "SELECT tablename from pg_tables WHERE tablename NOT LIKE 'pg_%%' AND tablename NOT LIKE 'sql_%%' ORDER BY tablename");
+ else
+ dmem_execute_printf(mem,c, &res, &row, &col, errstr, 0,HERE,
+		 "SELECT tablename from pg_tables ORDER BY tablename");
+ if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
+ if (col<0 || row<0) { debug("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
+ *ntabs = row;
+ data = (char**)dsmalloc(mem, row<<2, HERE);
+ for (i=0;i<row;i++) 
+   { 
+    data[i] = dsmalloc(mem, strlen(res[i][0])+1, HERE); 
+    strcpy(data[i], res[i][0]); 
+   }
+ dmem_free_p3c(mem, &res, row, col, HERE);
+ *dat = data;
+ return 1;
+}
+
 
 PGresult* execute_query(PGconn* c,  char* query)
 {
  PGresult* r;
  int stat;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d execute_query", __FILE__,__LINE__);
  debug("SQLquery: \"%s\"\n", query);
  if (!query)
    {
@@ -480,18 +663,117 @@ PGresult* execute_query(PGconn* c,  char* query)
  return r;
 }
 
-char*** mem_pgres2pc3(struct mem_list** mem, const PGresult* r, int* a, int* b, int want_meta)
+char*** dmem_pgres2pc3(struct dmem_list** mem, const PGresult* r, int* a, int* b, int want_meta, char* file, int line)
 {
  char*** result;
  char* tmp;
  int i,j;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d dmem_pgres2pc3 called from %s:%d", __FILE__,__LINE__,file,line);
  debug("mem_pgres2pc3: %p\n", (void*)r);
  if (!r) return 0;
  i=j=0;
  *a = PQntuples(r);
  *b = PQnfields(r);
+ if (!(*a) || !(*b)) 
+   { 
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
+    return NULL; 
+   }
  /*printf("ntuples=%d,%d\n", *a,*b);*/
+ /*if (!(*a) || !(*b)) { warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); return NULL; }*/
+ if (!want_meta)
+ {
+ result = (char***)dsmalloc(mem, (*a)<<2, HERE);
+ if (!result) { ERR error("smalloc error\n"); return NULL; }
+ for (i=0;i<*a;i++)
+   {
+    result[i] = (char**)dsmalloc(mem, (*b)<<2, HERE);
+    if (!result[i])
+      {
+       for (j=0;j<i;j++) dsfree(mem, (void**)(&result[i]), HERE);
+       ERR error("smalloc_i error!\n");
+       return NULL;
+      }
+   }
+ for (i=0;i<*a;i++)
+ for (j=0;j<*b;j++)
+   {
+    result[i][j] = (char*)dsmalloc(mem, PQgetlength(r,i,j)+1, HERE);
+    if (!result[i][j])
+      {
+       dmem_free_p3c(mem, &result,*a,*b, HERE);
+       ERR error("malloc_ij error\n");
+       return NULL;
+      }
+    strcpy(result[i][j], (char*)PQgetvalue(r,i,j));
+   }
+/*#ifdef DEBUG
+  debug("char*** result is (%d:%d):\n",i,j);
+  for (i=0;i<*a;i++)
+    {
+     for (j=0;j<*b;j++) debug("%-15s ", result[i][j]);
+     ln();
+    }
+#endif*/
+ return result;
+ }
+ else
+ {
+  if ((*b)<=0) return NULL;		/*FIXME is it OK? */
+ (*a)++;
+ result = (char***)dsmalloc(mem, (*a)<<2, HERE);
+ if (!result) { ERR error("smalloc error\n"); return NULL; }
+ for (i=0;i<*a;i++)
+   {
+    result[i] = (char**)dsmalloc(mem, (*b)<<2, HERE);
+    if (!result[i])
+      {
+       for (j=0;j<i;j++) dsfree(mem, (void**)(&result[i]), HERE);
+       ERR error("smalloc_i error!\n");
+       return NULL;
+      }
+   }
+ for (i=0;i<*b;i++)
+   {
+    tmp = PQfname(r,i);
+    result[0][i] = (char*)dsmalloc(mem, strlen(tmp)+1, HERE);
+    if (!result[0][i]) { ERR error("smalloc_namej error"); return NULL; }
+    strcpy(result[0][i], tmp);
+   }
+ for (i=1;i<*a;i++)
+ for (j=0;j<*b;j++)
+   {
+    result[i][j] = (char*)dsmalloc(mem, PQgetlength(r,i-1,j)+1, HERE);
+    if (!result[i][j])
+      {
+       dmem_free_p3c(mem, &result,*a,*b, HERE);
+       ERR error("malloc_ij error\n");
+       return NULL;
+      }
+    strcpy(result[i][j], (char*)PQgetvalue(r,i-1,j));
+   }
+  return result;
+ }
+}
+
+char*** mem_pgres2pc3(struct mem_list** mem, const PGresult* r, int* a, int* b, int want_meta)
+{
+ char*** result;
+ char* tmp;
+ int i,j;
+ debug("%s:%d mem_pgres2pc3", __FILE__,__LINE__);
+ debug("mem_pgres2pc3: %p\n", (void*)r);
+ if (!r) return 0;
+ i=j=0;
+ *a = PQntuples(r);
+ *b = PQnfields(r);
+ if (!(*a) || !(*b)) 
+   { 
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
+    return NULL; 
+   }
+ /*printf("ntuples=%d,%d\n", *a,*b);*/
+ /*if (!(*a) || !(*b)) { warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); return NULL; }*/
  if (!want_meta)
  {
  result = (char***)smalloc(mem, (*a)<<2);
@@ -567,28 +849,136 @@ char*** mem_pgres2pc3(struct mem_list** mem, const PGresult* r, int* a, int* b,
  }
 }
 
+char*** dmpgres2pc3(const PGresult* r, int* a, int* b, int want_meta, char* file, int line)
+{
+ char*** result;
+ char* tmp;
+ int i,j;
+ debug("%s:%d dmpgres2pc3 called from %s:%d", __FILE__,__LINE__,file,line);
+ debug("dmpgres2pc3: %p\n", (void*)r);
+ if (!r) return 0;
+ i=j=0;
+ *a = PQntuples(r);
+ *b = PQnfields(r);
+ /*if (!(*a) || !(*b)) { warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); return NULL; }*/
+ if (!(*a) || !(*b)) 
+   { 
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
+    return NULL; 
+   }
+ /*printf("ntuples=%d,%d\n", *a,*b);*/
+ if (!want_meta)
+ {
+ result = (char***)dmmalloc((*a)<<2, HERE);
+ if (!result) { ERR error("mmalloc error\n"); return NULL; }
+ for (i=0;i<*a;i++)
+  {
+    result[i] = (char**)dmmalloc((*b)<<2, HERE);
+    if (!result[i])
+      {
+       for (j=0;j<i;j++) 
+       {
+	 dmfree((void**)(&result[i]), HERE);
+       }
+       ERR error("mmalloc_i error!\n");
+       return NULL;
+      }
+   }
+ for (i=0;i<*a;i++)
+ for (j=0;j<*b;j++)
+   {
+    result[i][j] = (char*)dmmalloc(PQgetlength(r,i,j)+1, HERE);
+    if (!result[i][j])
+      {
+       dmfree_p3c(&result,*a,*b, HERE);
+       ERR error("mmalloc_ij error\n");
+       return NULL;
+      }
+    strcpy(result[i][j], (char*)PQgetvalue(r,i,j));
+   }
+/*#ifdef DEBUG
+  debug("char*** result is (%d:%d):\n",i,j);
+  for (i=0;i<*a;i++)
+    {
+     for (j=0;j<*b;j++) debug("%-15s ", result[i][j]);
+     ln();
+    }
+#endif*/
+ return result;
+ }
+ else
+ {
+  if ((*b)<=0) return NULL;		/*FIXME is it OK? */
+ (*a)++;
+ result = (char***)dmmalloc((*a)<<2, HERE);
+ if (!result) { ERR error("mmalloc error\n"); return NULL; }
+ for (i=0;i<*a;i++)
+   {
+    result[i] = (char**)dmmalloc((*b)<<2, HERE);
+    if (!result[i])
+      {
+       for (j=0;j<i;j++) 
+         {
+	  dmfree((void**)(&result[i]), HERE);
+	 }
+       ERR error("mmalloc_i error!\n");
+       return NULL;
+      }
+   }
+ for (i=0;i<*b;i++)
+   {
+    tmp = PQfname(r,i);
+    result[0][i] = (char*)dmmalloc(strlen(tmp)+1, HERE);
+    if (!result[0][i]) { ERR error("mmalloc_namej error"); return NULL; }
+    strcpy(result[0][i], tmp);
+   }
+ for (i=1;i<*a;i++)
+ for (j=0;j<*b;j++)
+   {
+    result[i][j] = (char*)dmmalloc(PQgetlength(r,i-1,j)+1, HERE);
+    if (!result[i][j])
+      {
+       dmfree_p3c(&result,*a,*b, HERE);
+       ERR error("mmalloc_ij error\n");
+       return NULL;
+      }
+    strcpy(result[i][j], (char*)PQgetvalue(r,i-1,j));
+   }
+  return result;
+ }
+}
+
 char*** mpgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
 {
  char*** result;
  char* tmp;
  int i,j;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d mpgres2pc3", __FILE__,__LINE__);
  debug("mpgres2pc3: %p\n", (void*)r);
  if (!r) return 0;
  i=j=0;
  *a = PQntuples(r);
  *b = PQnfields(r);
+ /*if (!(*a) || !(*b)) { warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); return NULL; }*/
+ if (!(*a) || !(*b)) 
+   { 
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
+    return NULL; 
+   }
  /*printf("ntuples=%d,%d\n", *a,*b);*/
  if (!want_meta)
  {
  result = (char***)mmalloc((*a)<<2);
  if (!result) { ERR error("mmalloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
-   {
+  {
     result[i] = (char**)mmalloc((*b)<<2);
     if (!result[i])
       {
-       for (j=0;j<i;j++) mfree((void**)(&result[i]));
+       for (j=0;j<i;j++) 
+       {
+	 mfree((void**)(&result[i]));
+       }
        ERR error("mmalloc_i error!\n");
        return NULL;
       }
@@ -626,7 +1016,10 @@ char*** mpgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
     result[i] = (char**)mmalloc((*b)<<2);
     if (!result[i])
       {
-       for (j=0;j<i;j++) mfree((void**)(&result[i]));
+       for (j=0;j<i;j++) 
+         {
+	  mfree((void**)(&result[i]));
+	 }
        ERR error("mmalloc_i error!\n");
        return NULL;
       }
@@ -659,12 +1052,17 @@ char*** pgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
  char*** result;
  char* tmp;
  int i,j;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d pgres2pc3", __FILE__,__LINE__);
  debug("pgres2pc3: %p\n", (void*)r);
  if (!r) return 0;
  i=j=0;
  *a = PQntuples(r);
  *b = PQnfields(r);
+ if (!(*a) || !(*b)) 
+   { 
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
+    return NULL; 
+   }
  /*printf("ntuples=%d,%d\n", *a,*b);*/
  if (!want_meta)
  {
@@ -744,7 +1142,7 @@ char*** pgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
 void mem_free_p3c(struct mem_list** mem, char**** p, int a, int b)
 {
  int i,j;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d mem_free_p3c", __FILE__,__LINE__);
  debug("mem_free_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
  if (!(*p)) return ;
  for (i=0;i<a;i++) if (!(*p)[i]) return ;
@@ -754,10 +1152,23 @@ void mem_free_p3c(struct mem_list** mem, char**** p, int a, int b)
  sfree(mem, (void**)(&(*p)));
 }
 
+void dmem_free_p3c(struct dmem_list** mem, char**** p, int a, int b, char* file, int line)
+{
+ int i,j;
+ debug("%s:%d dmem_free_p3c called from %s:%d", __FILE__,__LINE__,file,line);
+ debug("dmem_free_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
+ if (!(*p)) return ;
+ for (i=0;i<a;i++) if (!(*p)[i]) return ;
+ for (i=0;i<a;i++)
+ for (j=0;j<b;j++) if ((*p)[i][j]) dsfree(mem, (void**)(&(*p)[i][j]), HERE);
+ for (i=0;i<a;i++) if ((*p)[i])    dsfree(mem, (void**)(&(*p)[i]), HERE);
+ dsfree(mem, (void**)(&(*p)), HERE);
+}
+
 void mfree_p3c(char**** p, int a, int b)
 {
  int i,j;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d mfree_p3c", __FILE__,__LINE__);
  debug("mfree_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
  if (!(*p)) return ;
  for (i=0;i<a;i++) if (!(*p)[i]) return ;
@@ -767,10 +1178,23 @@ void mfree_p3c(char**** p, int a, int b)
  mfree((void**)(&(*p)));
 }
 
+void dmfree_p3c(char**** p, int a, int b, char* file, int line)
+{
+ int i,j;
+ debug("%s:%d dmfree_p3c called from: %s:%d", __FILE__,__LINE__,file,line);
+ debug("dmfree_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
+ if (!(*p)) return ;
+ for (i=0;i<a;i++) if (!(*p)[i]) return ;
+ for (i=0;i<a;i++)
+ for (j=0;j<b;j++) if ((*p)[i][j]) dmfree((void**)(&(*p)[i][j]), HERE);
+ for (i=0;i<a;i++) if ((*p)[i])    dmfree((void**)(&(*p)[i]), HERE);
+ dmfree((void**)(&(*p)), HERE);
+}
+
 void free_p3c(char**** p, int a, int b)
 {
  int i,j;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d free_p3c", __FILE__,__LINE__);
  debug("free_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
  if (!(*p)) return ;
  for (i=0;i<a;i++) if (!(*p)[i]) return ;
