@@ -628,7 +628,7 @@ static char *xpm_big[] =
 
 int about_clback(GtkWidget* w, gpointer d)
 {
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d about_clback", __FILE__,__LINE__);
  gtk_main_quit();
  return FALSE;
 }
@@ -641,7 +641,7 @@ void about_window_new()
  GtkStyle *style;
  GdkColormap* gcm;
  GdkBitmap *mask;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d about_window_new", __FILE__,__LINE__);
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_modal(GTK_WINDOW(window), 1);
  gtk_widget_set_usize (GTK_WIDGET(window), 400, 300);
@@ -693,7 +693,7 @@ void get_toolbar(GtkWidget* w, GtkWidget **tb)
  GtkStyle *style;
  GtkWidget* pix;
  GdkColormap* gcm;
- debug("%s:%d", __FILE__,__LINE__);
+ debug("%s:%d get_toolbar", __FILE__,__LINE__);
  *tb = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gtk_toolbar_set_tooltips(GTK_TOOLBAR(*tb), TRUE);
  gtk_container_set_border_width(GTK_CONTAINER(*tb), 5);
